@@ -1,15 +1,15 @@
- /*  X-automalab, a totalistic celular automaton simulator for X windows.
+ /*  X-automalab, a totalistic cellular automaton simulator for X windows.
            
       Michael Creutz   
-  creutz@wind.phy.bnl.gov 
+  creutz@bnl.gov 
 
  compiling:  cc -O -o xautomalab -L/usr/X11R6/lib xautomalab.c -lX11 
 
- version 2.0 
- 24 Nov 1995 
- LZW disabled, 10 October 1998
+ version 2.1
+ August 2006
 
  minor modifications to eliminate gcc -Wall warnings, March 1999
+ left-right error depending on bit order fixed, June 1999
 
  This program is still evolving; the latest release can be found via
 
@@ -25,12 +25,9 @@
  1.7 prettier buttons and colors 
  1.8 fixed a color table error in the gif save routine
  2.0 adds speed gadget (it seemed too fast on an alpha)   
+ 2.1 minor fixes; lzw restored
  */
 
-/* because of the unisys patent on LZW, it may be illegal to
-   distribute this program with the following "if" changed to "while" */
-#define LZW if
-
 # include <X11/Xlib.h>
 # include <X11/Xutil.h>
 # include <X11/Xos.h>
@@ -46,15 +43,18 @@
 /* size and position parameters for buttons, etc. */
 # define PLAYTOP 130
 # define PLAYLEFT 10
-# define BOUNDHEIGHT 72
-# define TRACEHEIGHT 36
-# define BOUNDWIDTH  112
 # define BARHEIGHT 45
 # define BARWIDTH  120
 # define BARTOP 2
 # define BARLEFT 2
-# define XHEIGHT 36
 
+# define BUTTONWIDTH  112
+# define BUTTONHEIGHT 18
+# define RIGHTBUTTONS 11
+# define RIGHTBOXHEIGHT (BUTTONHEIGHT*RIGHTBUTTONS)
+# define XHEIGHT (2*BUTTONHEIGHT)
+
+static char *progname;
 int nrows,ncols,volume; /* lattice dimensions (plus two for boundaries) */
      /* ncols will be truncated to 
         a multiple of bytes per long word (BPW),
@@ -85,28 +85,32 @@ unsigned char *work=NULL;            /* points to a work array */
 /* various window stuff */
 Display *display;
 int screen;
-static char *progname;
-Window window,quitbutton,pausebutton,playground,boundwindow,rulebar,
-       fillbutton,tracewindow,xpastbutton,nhood,srbutton,blockbutton,
-       speedbutton,makebutton();
+Window window,quitbutton,pausebutton,playground,rulebar,
+       fillbutton,nhood,rightbox,
+       srbutton,speedbutton,makebutton();
 XColor xcolor,colorcell;
 Colormap cmap;
 GC gc,gcpen;
-int windowwidth,windowheight;
 XFontStruct *font=NULL;
-int font_height,font_width;
 XSizeHints size_hints;
-int darkcolor,lightcolor,black,white;
 XImage *spinimage=NULL;
+
+
+int windowwidth,windowheight;
+int font_height,font_width;
+int darkcolor,lightcolor,black,white;
 long translate[256];                /* for converting colors   */
-char * boundtext[4]={"open","live","periodic","flow"};
+
+/* text for buttons */
+char * righttext[RIGHTBUTTONS]={"open","live","periodic","flow",
+"boundary","tracer","clear trace","xor past","","","cell size"};
 
 void settable(),openwindow(),makebuttons(),update(),repaint(),cleanup(),
      fixboundary(),showpic(),drawbutton(),setboundary(),
      settrace(),setrule(),sketch(),setnbrhd(),savepic(),loadpic(),
      setxpast(),putneighbors();
 
-void main(argc,argv)
+int main(argc,argv)
 int argc;
 char **argv;
 {unsigned int width, height;
@@ -115,7 +119,7 @@ char **argv;
  XEvent report;
  progname=argv[0];
 
-  /* initial lattice size */
+ /* initial lattice size */
  block=1;
  nrows=200/block;
  ncols=(~(BPW-1))&(200/block);
@@ -123,8 +127,8 @@ char **argv;
 
  /* initial rule */
  for (i=0;i<3;i++)
-  for (j=0;j<3;j++)
-   nbrhd[i][j]=1;
+   for (j=0;j<3;j++)
+     nbrhd[i][j]=1;
  nbrhd[0][0]=nbrhd[2][1]=0;
  trace=1;
  xpast=0;
@@ -137,145 +141,155 @@ char **argv;
  openwindow(argc,argv);
  makebuttons();
 
-  /* set the masks for parallel updating */
+ /* set the masks for parallel updating */
  for (i=0;i<sizeof(long);i++) 
-  {mask0x13=0x13|(mask0x13<<8);
+   {mask0x13=0x13|(mask0x13<<8);
    mask0x01=0x01|(mask0x01<<8);
    mask0x03=0x03|(mask0x03<<8);
    mask0x10=0x10|(mask0x10<<8);
-  }
-   /* check for byte order in a word */
+   }
+ /* check for byte order in a word */
  bitcheck=1;
  if (* (char*) (&bitcheck) )
    colshift=(-1); /* LSBfirst */
  else 
    colshift=1;    /* MSBfirst */
 
-/* loop forever, looking for events */
+ /* loop forever, looking for events */
  while(1)
-  {if ((0==XPending(display))&&(0==paused))
-    {if (delay) /* don't update yet */
+   {if ((0==XPending(display))&&(0==paused))
+     {if (delay) /* don't update yet */
        {delay--;
-        /* this use of select() seems a kludge to me; 
-           why can't usleep() be more standard? */
-        timeout.tv_sec=0;
-        timeout.tv_usec=100000; /* .1 sec per delay unit */
-        select(0,NULL,NULL,NULL,&timeout); 
+       /* this use of select() seems a kludge to me; 
+	  why can't usleep() be more standard? */
+       timeout.tv_sec=0;
+       timeout.tv_usec=100000; /* .1 sec per delay unit */
+       select(0,NULL,NULL,NULL,&timeout); 
        }
      else
-      {delay=speed;
+       {delay=speed;
        update();
-      }
-    }
+       }
+     }
    else
-    {XNextEvent(display,&report); 
+     {XNextEvent(display,&report); 
      switch (report.type)
-      {case Expose:
-        if ((report.xexpose.window)!=window) break; /* cuts down flashing,
-           but you might remove this line if things aren't being redrawn */
-        if (report.xexpose.count!=0) break; /* more in queue, wait for them */
-        repaint();  
-        break;
+       {case Expose:
+	  if ((report.xexpose.window)!=window) break; 
+	  /* cuts down flashing, but you might remove this
+	     if things aren't being redrawn */
+	  if (report.xexpose.count!=0) break; /* more in queue, wait */
+	  repaint();  
+	  break;
        case ConfigureNotify:
-        width=report.xconfigure.width;
-        height=report.xconfigure.height;
-        if ((width<size_hints.min_width)||(height<size_hints.min_height))
-            {fprintf(stderr,"%s: window too small to proceed.\n",progname);
-             cleanup();
-            } 
-        else if ((width!=windowwidth)||(height!=windowheight))
-            {windowwidth=width;
-             windowheight=height;  
-             nrows=(height-PLAYTOP-14)/block;
-             ncols=(~(BPW-1))&((width-PLAYLEFT-BOUNDWIDTH-20)/block);
-             volume=nrows*ncols;
-             makebuttons();
-             fixboundary();
-             showpic();
-            } 
-        break; 
+	 width=report.xconfigure.width;
+	 height=report.xconfigure.height;
+	 if ((width<size_hints.min_width)||(height<size_hints.min_height))
+	   {fprintf(stderr,"%s: window too small to proceed.\n",progname);
+	   cleanup();
+	   } 
+	 else if ((width!=windowwidth)||(height!=windowheight))
+	   {windowwidth=width;
+	   windowheight=height;  
+	   nrows=(height-PLAYTOP-14)/block;
+	   ncols=(~(BPW-1))&((width-PLAYLEFT-BUTTONWIDTH-20)/block);
+	   volume=nrows*ncols;
+	   XClearWindow(display,window);
+	   makebuttons();
+	   fixboundary();
+	   repaint();
+	   } 
+	 break; 
        case ButtonPress:
-        if (report.xbutton.window==quitbutton)
-             cleanup();
-        else if (report.xbutton.window==pausebutton)
-            {paused=1-paused;
-             drawbutton(pausebutton,0,0,68,18,"pause",1-2*paused);
-            } 
-        else if (report.xbutton.window==fillbutton)
-            {drawbutton(fillbutton,0,0, 68,18,"clear",-1);
-             for (i=0;i<volume;i++)
-               field[old][i]=0;
-             fixboundary();
+	 if (report.xbutton.window==quitbutton)
+	   cleanup();
+	 else if (report.xbutton.window==pausebutton)
+	   {paused=1-paused;
+	   drawbutton(pausebutton,0,0,68,18,"pause",1-2*paused);
+	   } 
+	 else if (report.xbutton.window==fillbutton)
+	   {drawbutton(fillbutton,0,0, 68,18,"clear",-1);
+	   for (i=0;i<volume;i++)
+	     field[old][i]=0;
+	   fixboundary();
+	   showpic();
+	   drawbutton(fillbutton,0,0, 68,18,"clear",1);
+	   }
+	 else if (report.xbutton.window==rightbox){
+	   y=report.xbutton.y/BUTTONHEIGHT;
+	   if (y<4)
+	     {setboundary(y);
              showpic();
-             drawbutton(fillbutton,0,0, 68,18,"clear",1);
-            }
-        else if (report.xbutton.window==boundwindow)
-            {setboundary((report.xbutton.y*4)/BOUNDHEIGHT);
-             showpic();
-            }
-        else if (report.xbutton.window==tracewindow)
-             settrace((report.xbutton.y*2)/TRACEHEIGHT);
-        else if (report.xbutton.window==rulebar)
-             setrule((report.xbutton.x*(neighbors+1))/BARWIDTH+
+	     }
+	   else if (y==5)
+             settrace(0);
+	   else if (y==6)
+             settrace(1);
+	   else if (y==7)
+	     setxpast(0);
+	   else if (y==8)
+	     setxpast(1);
+	   else if (y==9)
+	     {i=(1<<(4*report.xbutton.x/BUTTONWIDTH));
+	     if (i!=block) /* reset for new block size */
+	       {block=i;
+	       nrows=(windowheight-PLAYTOP-10)/block;
+	       ncols=(~(BPW-1))&((windowwidth-PLAYLEFT-BUTTONWIDTH-20)/block);
+	       volume=nrows*ncols;
+	       /* clear old window */
+	       XSetForeground(display,gcpen,lightcolor);  
+	       XFillRectangle(display,window,gcpen,0,0,
+			      windowwidth,windowheight);
+	       makebuttons();
+	       fixboundary();
+	       showpic();
+	       }
+	     }
+	 }
+
+	 else if (report.xbutton.window==rulebar)
+	   setrule((report.xbutton.x*(neighbors+1))/BARWIDTH+
                    ((report.xbutton.y*3)/BARHEIGHT)*10);
-        else if (report.xbutton.window==playground) /* sketch */
-            {x=report.xbutton.x/block;
-             y=report.xbutton.y/block;
-             sketch(x,y);
-            }
-        else if (report.xbutton.window==xpastbutton) 
-             setxpast((report.xbutton.y*2)/XHEIGHT);
-        else if (report.xbutton.window==nhood) 
-            {x=(3*report.xbutton.x)/BARHEIGHT;
-             y=(3*report.xbutton.y)/BARHEIGHT;
-             setnbrhd(x,y);
-            }
-        else if (report.xbutton.window==srbutton)
-            {if (2*report.xbutton.y<XHEIGHT)
-               {drawbutton(srbutton,0,0,68,18,"save",-1);
-                savepic(field[old],ncols,nrows);
-                drawbutton(srbutton,0,0,68,18,"save",1);
-	       }
-             else
-               {drawbutton(srbutton,0,XHEIGHT/2,68,18,"restore",-1);
-                loadpic(field[old],ncols,nrows);
-                drawbutton(srbutton,0,XHEIGHT/2,68,18,"restore",1);
-	       }
-            }
-        else if (report.xbutton.window==blockbutton)
-          {i=(1<<(4*report.xbutton.x/BOUNDWIDTH));
-           if (i!=block) /* reset for new block size */
-             {block=i;
-              nrows=(windowheight-PLAYTOP-10)/block;
-              ncols=(~(BPW-1))&((windowwidth-PLAYLEFT-BOUNDWIDTH-20)/block);
-              volume=nrows*ncols;
-                 /* clear old window */
-              XSetForeground(display,gcpen,lightcolor);  
-              XFillRectangle(display,window,gcpen,0,0,
-                             windowwidth,windowheight);
-              makebuttons();
-              fixboundary();
-              showpic();
-             }
-          }
-        else if (report.xbutton.window==speedbutton)
-          {  /* reset speed */
-           speed=10-(11*report.xbutton.x)/BOUNDWIDTH;
-           if (speed<0) speed=0;
-           if (speed>10) speed=10;
-           delay=speed;
-           drawbutton(speedbutton,0,0,BOUNDWIDTH,18,"speed",-1);
-           drawbutton(speedbutton,1+((10-speed)*(BOUNDWIDTH-2))/11,1,
-                      (BOUNDWIDTH-2)/11,16,"",2);
-          }
-        else
-         update(); /* do a sweep when mouse clicked */
-        break;
+	 else if (report.xbutton.window==playground) /* sketch */
+	   {x=report.xbutton.x/block;
+	   y=report.xbutton.y/block;
+	   sketch(x,y);
+	   }
+	 else if (report.xbutton.window==nhood) 
+	   {x=(3*report.xbutton.x)/BARHEIGHT;
+	   y=(3*report.xbutton.y)/BARHEIGHT;
+	   setnbrhd(x,y);
+	   }
+	 else if (report.xbutton.window==srbutton)
+	   {if (2*report.xbutton.y<XHEIGHT)
+	     {drawbutton(srbutton,0,0,68,18,"save",-1);
+	     savepic(field[old],ncols,nrows);
+	     drawbutton(srbutton,0,0,68,18,"save",1);
+	     }
+	   else
+	     {drawbutton(srbutton,0,XHEIGHT/2,68,18,"restore",-1);
+	     loadpic(field[old],ncols,nrows);
+	     drawbutton(srbutton,0,XHEIGHT/2,68,18,"restore",1);
+	     }
+	   }
+	 else if (report.xbutton.window==speedbutton)
+	   {  /* reset speed */
+	     speed=10-(11*report.xbutton.x)/BUTTONWIDTH;
+	     if (speed<0) speed=0;
+	     if (speed>10) speed=10;
+	     delay=speed;
+	     drawbutton(speedbutton,0,0,BUTTONWIDTH,18,"speed",-1);
+	     drawbutton(speedbutton,1+((10-speed)*(BUTTONWIDTH-2))/11,1,
+			(BUTTONWIDTH-2)/11,16,"",2);
+	   }
+	 else
+	   update(); /* do a sweep when mouse clicked */
+	 break;
        default:
-        break;
-      } /* end of switch */
-    } /* end of if XPending */
-  } /* end of while(1) */
+	 break;
+       } /* end of switch */
+     } /* end of if XPending */
+   } /* end of while(1) */
 } /* end of main */
 
 void settable()  /* set up the rule table from birth and survivor arrays */
@@ -284,6 +298,7 @@ void settable()  /* set up the rule table from birth and survivor arrays */
    Any ideas as to how to add easily selected further rules? */
 {int i,j,bits,mask;
   /* mask out the unused neighbors */
+ if (colshift>0) /* correct for byte order */
  mask=nbrhd[0][1]
      |nbrhd[2][1]<<1
      |nbrhd[1][2]<<2
@@ -292,6 +307,15 @@ void settable()  /* set up the rule table from birth and survivor arrays */
      |nbrhd[2][2]<<5
      |nbrhd[0][0]<<6
      |nbrhd[2][0]<<7;
+ else
+ mask=nbrhd[2][1]
+     |nbrhd[0][1]<<1
+     |nbrhd[1][2]<<2
+     |nbrhd[1][0]<<3
+     |nbrhd[2][2]<<4
+     |nbrhd[0][2]<<5
+     |nbrhd[2][0]<<6
+     |nbrhd[0][0]<<7;
  for (i=0;i<256;i++)
   {bits=0;  /* count the set bits in i */
    j=i&mask;
@@ -305,8 +329,7 @@ void settable()  /* set up the rule table from birth and survivor arrays */
  return;
 }
 
-void sketch(x,y)   /* sketch in playground starting at point x,y */
-int x,y;
+void sketch(int x, int y)   /* sketch in playground starting at point x,y */
 {int sketching,i,oldx,oldy,deltax,deltay,newx,newy;
  Window root,child;
  XEvent sketchreport;
@@ -371,10 +394,9 @@ void update()
  return;
 }
 
-void putneighbors(source,destination)
+void putneighbors(long *source, long *destination)
 /* initializes the new field and puts neighbors in work array */
 /* this updating is done BPW spins at a time using multi-spin coding */
-long *source, *destination;
 {int i,rowshift=ncols/BPW,topi,cprshift,cmrshift;
  unsigned long * temp;
  temp=(unsigned long*) work;
@@ -471,6 +493,7 @@ void showpic() /* display the field */
      }
   }
  XPutImage(display,playground,gc,spinimage,0,0,0,0,block*ncols,block*nrows); 
+ XSync(display,0);
  return;
 }
 
@@ -518,101 +541,90 @@ void repaint()
  drawbutton(srbutton,  0,0, 68,18,"save",1);
  drawbutton(srbutton,  0,18,68,18,"restore",1);
  drawbutton(pausebutton,0,0,68,18,"pause",1-2*paused);
- drawbutton(speedbutton,0,0,BOUNDWIDTH,18,"speed",-1);
- drawbutton(speedbutton,1+((10-speed)*(BOUNDWIDTH-2))/11,1,
-               (BOUNDWIDTH-2)/11,16,"",2);
+ drawbutton(speedbutton,0,0,BUTTONWIDTH,18,"speed",-1);
+ drawbutton(speedbutton,1+((10-speed)*(BUTTONWIDTH-2))/11,1,
+	    (BUTTONWIDTH-2)/11,16,"",2);
+
  drawbutton(window,PLAYLEFT-4,PLAYTOP-4,block*ncols+8,block*nrows+8,NULL,4);
- setxpast(-1);
  setnbrhd(-1,-1);
 
-  /* fix the block buttons */
+ /* draw the boundary condition buttons etc */
+ for (i=0;i<RIGHTBUTTONS;i++)
+   drawbutton(rightbox,0,i*BUTTONHEIGHT,BUTTONWIDTH,BUTTONHEIGHT,righttext[i],
+	      ((i!=4)&&(i!=10)&&(i!=7)) /* buttons without borders */
+	      *(1-2*((i==boundary)||(i==5)))); /* press boundary and tracer */
+
+ setxpast(-1);
+
+
+ /* fix the block buttons */
  for (i=1;i<=4;i++)
-  {
-   sprintf(stringbuffer,"%d",(1<<i)/2);
-   drawbutton(blockbutton,(i-1)*BOUNDWIDTH/4,0,
-         BOUNDWIDTH/4,18,
-         stringbuffer,1-2*(i==(blockshift+1)));
-  }
+   {
+     sprintf(stringbuffer,"%d",(1<<i)/2);
+     drawbutton(rightbox,(i-1)*BUTTONWIDTH/4,9*BUTTONHEIGHT,
+		BUTTONWIDTH/4,BUTTONHEIGHT,
+		stringbuffer,1-2*(i==(blockshift+1)));
+   }
 
-  /* draw the boundary condition buttons */
- for (i=0;i<4;i++)
-  drawbutton(boundwindow,0,i*BOUNDHEIGHT/4,BOUNDWIDTH,BOUNDHEIGHT/4,
-     boundtext[i],1-2*(i==boundary));
-
-  /* draw the trace buttons */
- drawbutton(tracewindow,0,0,BOUNDWIDTH,TRACEHEIGHT/2,
-        "tracer",1-2*trace);
- drawbutton(tracewindow,0,TRACEHEIGHT/2,BOUNDWIDTH,TRACEHEIGHT/2,
-        "clear trace",1);
-
-          /* write various strings */
+ /* write various strings */
  XDrawString(display,window,gc,BARWIDTH+BARLEFT+2,BARTOP+font_height-2,
-        "births",6);
+	     "births",6);
  XDrawString(display,window,gc,BARWIDTH+BARLEFT+2,BARTOP+BARHEIGHT-2,
-        "survivors",9);
+	     "survivors",9);
  XDrawString(display,window,gc,70,BARHEIGHT+BARTOP+2*font_height,
-        "neighborhood",12);
+	     "neighborhood",12);
  sprintf(stringbuffer,"%d by %d lattice    ",ncols-2,nrows-2);
  XDrawImageString(display,window,gc,5,BARHEIGHT+BARTOP+5*font_height,
-               stringbuffer,strlen(stringbuffer));
+		  stringbuffer,strlen(stringbuffer));
  if ((*spinimage).depth>1)
-  {XSetForeground(display,gcpen,translate[1]);
+   {XSetForeground(display,gcpen,translate[1]);
    XDrawString(display,window,gcpen,5,BARTOP+BARHEIGHT+4*font_height,
-         "young",5); 
+	       "young",5); 
    XSetForeground(display,gcpen,translate[3]);
    XDrawString(display,window,gcpen,60,BARTOP+BARHEIGHT+4*font_height,
-         "old",3); 
+	       "old",3); 
    XSetForeground(display,gcpen,translate[2]);
    XDrawString(display,window,gcpen,98,BARTOP+BARHEIGHT+4*font_height,
-         "moribund",8); 
-  }
- XDrawString(display,window,gc,windowwidth-BOUNDWIDTH,BOUNDHEIGHT+font_height,
-               "  Boundary  ",12);
- XDrawString(display,window,gc,
-        windowwidth-BOUNDWIDTH+12,BOUNDHEIGHT+TRACEHEIGHT+XHEIGHT+76,
-        "cell size",9);
- XDrawString(display,window,gc,windowwidth-50,windowheight-5,"MJC",3); 
+	       "moribund",8); 
+   }
  showpic();  
  return;
 }
  
-void setboundary(value)
+void setboundary(int value)
 /* take action if boundary buttons hit */
-int value;
-{drawbutton(boundwindow,0,(BOUNDHEIGHT*boundary)/4,BOUNDWIDTH,BOUNDHEIGHT/4,
-                       boundtext[boundary],1);
+{drawbutton(rightbox,0,(BUTTONHEIGHT*boundary),BUTTONWIDTH,BUTTONHEIGHT,
+                       righttext[boundary],1);
  boundary=value;
- drawbutton(boundwindow,0,(BOUNDHEIGHT*boundary)/4,BOUNDWIDTH,BOUNDHEIGHT/4,
-                       boundtext[boundary],-1);
+ drawbutton(rightbox,0,(BUTTONHEIGHT*boundary),BUTTONWIDTH,BUTTONHEIGHT,
+                       righttext[boundary],-1);
  fixboundary();
  return;
 }
  
-void settrace(value)
+void settrace(int value)
 /* take action if trace buttons hit */
-int value;
 {int i;
  if (1==value) /* clear the trace bit */
-  {drawbutton(tracewindow,0,TRACEHEIGHT/2,BOUNDWIDTH,TRACEHEIGHT/2,
+  {drawbutton(rightbox,0,6*BUTTONHEIGHT,BUTTONWIDTH,BUTTONHEIGHT,
         "clear trace",-1);
    for (i=0;i<volume;i++)
      field[old][i]&=0x3;
    showpic();
-   drawbutton(tracewindow,0,TRACEHEIGHT/2,BOUNDWIDTH,TRACEHEIGHT/2,
+   drawbutton(rightbox,0,6*BUTTONHEIGHT,BUTTONWIDTH,BUTTONHEIGHT,
         "clear trace",1);
   } 
  else 
   {trace=1-trace;
-   drawbutton(tracewindow,0,0,BOUNDWIDTH,TRACEHEIGHT/2,
+   drawbutton(rightbox,0,5*BUTTONHEIGHT,BUTTONWIDTH,BUTTONHEIGHT,
           "tracer",1-2*trace);
   }
  return;
 }
 
-void setrule(value)
+void setrule(int value)
 /* take action if rule buttons hit */
 /* call with negative value just to redraw but not change rule */
-int value;
 {int i;  
  if (value>=0) 
    {if (value<10)
@@ -637,8 +649,7 @@ int value;
  return;
 }
 
-void setnbrhd(vx,vy)  /* fix the neighborhood buttons */
-int vx,vy;
+void setnbrhd(int vx, int vy)  /* fix the neighborhood buttons */
 {int i,j;
  neighbors=0;
  if ((vx>=0)&(vy>=0))  /* toggle this neighbor */
@@ -654,33 +665,30 @@ int vx,vy;
  return;
 }
 
-void setxpast(value)   /* fix up the xpast button */
-int value;
+void setxpast(int value)   /* fix up the xpast button */
 {int i;
  if ((1==value)&xpast) /* swap past and present */
-  {drawbutton(xpastbutton,0,XHEIGHT/2,BOUNDWIDTH,XHEIGHT/2,"reverse",-1);
+  {drawbutton(rightbox,0,8*BUTTONHEIGHT,BUTTONWIDTH,XHEIGHT/2,"reverse",-1);
    for (i=0;i<volume;i++)
     field[old][i]=((field[old][i]>>1)&1)|((field[old][i]&1)<<1);
    showpic();
-   drawbutton(xpastbutton,0,XHEIGHT/2,BOUNDWIDTH,XHEIGHT/2,"reverse",1);
+   drawbutton(rightbox,0,8*BUTTONHEIGHT,BUTTONWIDTH,XHEIGHT/2,"reverse",1);
   }
- if ((0==value)|((1==value)&(0==xpast))) xpast=1-xpast;
+ if (0==value) xpast=1-xpast;
  /* repaint the button */
  if (xpast) 
-  {drawbutton(xpastbutton,0,0,BOUNDWIDTH,XHEIGHT/2,"xor past",-1);
-   drawbutton(xpastbutton,0,XHEIGHT/2,BOUNDWIDTH,XHEIGHT/2,"reverse",1);
+  {drawbutton(rightbox,0,7*BUTTONHEIGHT,BUTTONWIDTH,XHEIGHT/2,"xor past",-1);
+   drawbutton(rightbox,0,8*BUTTONHEIGHT,BUTTONWIDTH,XHEIGHT/2,"reverse",1);
   }
  else
-  {drawbutton(xpastbutton,0,0,BOUNDWIDTH,XHEIGHT/2,"xor past",1);
-   drawbutton(xpastbutton,0,XHEIGHT/2,BOUNDWIDTH,XHEIGHT/2,NULL,0);
+  {drawbutton(rightbox,0,7*BUTTONHEIGHT,BUTTONWIDTH,XHEIGHT/2,"xor past",1);
+   drawbutton(rightbox,0,8*BUTTONHEIGHT,BUTTONWIDTH,XHEIGHT/2,NULL,0);
   }
  return;
 }
 
-void openwindow(argc,argv)
+void openwindow(int argc, char **argv)
 /* a lot of this is taken from basicwin in the Xlib Programming Manual */
-int argc;
-char **argv;
 {char *window_name="Automalab";
  char *icon_name="Automalab";
  long event_mask;
@@ -689,7 +697,7 @@ char **argv;
  int i;
 # define icon_bitmap_width 16
 # define icon_bitmap_height 16
- static unsigned char icon_bitmap_bits[] = {
+ static char icon_bitmap_bits[] = {
    0x1f, 0xf8, 0x1f, 0x88, 0x1f, 0x88, 0x1f, 0x88, 0x1f, 0x88, 0x1f, 0xf8,
    0x1f, 0xf8, 0x1f, 0xf8, 0x1f, 0xf8, 0x1f, 0xf8, 0x1f, 0xf8, 0xff, 0xff,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
@@ -747,7 +755,7 @@ if (DefaultDepth(display,screen)>1)
  translate[16]=translate[8]; /* for tracing */
 
    /* make the main window */
- windowwidth=(block*ncols+PLAYLEFT+BOUNDWIDTH+20);
+ windowwidth=(block*ncols+PLAYLEFT+BUTTONWIDTH+20);
  windowheight=(PLAYTOP+block*nrows+14);
  window=XCreateSimpleWindow(display,RootWindow(display,screen),
    0,0,windowwidth,windowheight,4,black,lightcolor);
@@ -833,109 +841,98 @@ void makebuttons()
  long event_mask;
  XEvent report;
  Cursor cursor;
-     /* first destroy any old buttons */
+ /* first destroy any old buttons */
  XDestroySubwindows(display,window);
-     /* now make the new buttons */
+ /* now make the new buttons */
  quitbutton =makebutton(windowwidth-92,windowheight-42 ,68,18);
  pausebutton=makebutton(windowwidth-92,windowheight-60 ,68,18);
  fillbutton =makebutton(windowwidth-92,windowheight-78 ,68,18);
  srbutton   =makebutton(windowwidth-92,windowheight-114,68,36);
- speedbutton=makebutton(windowwidth-BOUNDWIDTH-4,
-           BOUNDHEIGHT+TRACEHEIGHT+XHEIGHT+24,
-           BOUNDWIDTH,18);
+ speedbutton=makebutton(windowwidth-BUTTONWIDTH-4,
+			RIGHTBOXHEIGHT+8,BUTTONWIDTH,BUTTONHEIGHT);
  rulebar    =makebutton(BARLEFT,BARTOP,BARWIDTH,BARHEIGHT);
  nhood      =makebutton(20,BARHEIGHT+BARTOP+4,BARHEIGHT,BARHEIGHT);
- boundwindow=makebutton(windowwidth-BOUNDWIDTH-4,2,
-                        BOUNDWIDTH,BOUNDHEIGHT);
- tracewindow=makebutton(windowwidth-BOUNDWIDTH-4,BOUNDHEIGHT+20,
-                        BOUNDWIDTH,TRACEHEIGHT);
- xpastbutton=makebutton(windowwidth-BOUNDWIDTH-4,BOUNDHEIGHT+TRACEHEIGHT+22,
-                        BOUNDWIDTH,XHEIGHT);
- blockbutton=makebutton(windowwidth-BOUNDWIDTH-4,
-            BOUNDHEIGHT+TRACEHEIGHT+XHEIGHT+46,BOUNDWIDTH,18);
+ rightbox   =makebutton(windowwidth-BUTTONWIDTH-4,2,
+                        BUTTONWIDTH,RIGHTBOXHEIGHT);
 
  playground=XCreateSimpleWindow(display,window,
-    PLAYLEFT,PLAYTOP,block*ncols,block*nrows,0,black,translate[4]);
+				PLAYLEFT,PLAYTOP,block*ncols,
+				block*nrows,0,black,translate[4]);
  event_mask=ExposureMask|ButtonReleaseMask|ButtonPressMask|
-            PointerMotionHintMask|ButtonMotionMask; 
+   PointerMotionHintMask|ButtonMotionMask; 
  XSelectInput(display,playground,event_mask);
  XMapWindow(display,playground);
 
-   /* wait for playground to be displayed befor proceeding */
+ /* wait for playground to be displayed befor proceeding */
  i=1; /* a flag */
  while (i)
-    {XNextEvent(display,&report); 
-     switch (report.type)
-      {case Expose:
-          if (report.xexpose.window!=playground) i=0;
-       default:
-        break;
-      }
-    }
+   {XNextEvent(display,&report); 
+   switch (report.type)
+     {case Expose:
+	if (report.xexpose.window!=playground) i=0;
+     default:
+       break;
+     }
+   }
 
-   /* make image structure, first destroying any old one */
+ /* make image structure, first destroying any old one */
  if (NULL!=spinimage) 
-  {XDestroyImage(spinimage); 
+   {XDestroyImage(spinimage); 
    spinimage=NULL;
-  }
+   }
  spinimage=XGetImage((Display *) display, (Drawable) playground,
-           0,0,block*ncols,block*nrows,
-           AllPlanes,ZPixmap);
+		     0,0,block*ncols,block*nrows,
+		     AllPlanes,ZPixmap);
  if (NULL==spinimage)
-       {fprintf(stderr,"trouble creating image structure\n");
-        exit(-1);
-       } 
+   {fprintf(stderr,"trouble creating image structure\n");
+   exit(-1);
+   } 
 
-  /* make special cursors to be cute */
+ /* make special cursors to be cute */
  cursor=XCreateFontCursor(display,XC_sb_up_arrow); 
  XDefineCursor(display,playground,cursor);
- cursor=XCreateFontCursor(display,XC_exchange);
- XDefineCursor(display,xpastbutton,cursor);
  cursor=XCreateFontCursor(display,XC_hand2);
+ XDefineCursor(display,rightbox,cursor);
  XDefineCursor(display,nhood,cursor);
  XDefineCursor(display,rulebar,cursor);
- XDefineCursor(display,boundwindow,cursor);
- XDefineCursor(display,tracewindow,cursor);
  XDefineCursor(display,srbutton,cursor);
  XDefineCursor(display,quitbutton,cursor);
  XDefineCursor(display,pausebutton,cursor);
  XDefineCursor(display,fillbutton,cursor);
- XDefineCursor(display,blockbutton,cursor);
  XDefineCursor(display,speedbutton,cursor);
 
-   /* reallocate various arrays */
+ /* reallocate various arrays */
  if (NULL!=work) free((char *) work);
  if (NULL==( work=(unsigned char *) malloc(volume)))
-     {fprintf(stderr,"allocation problems\n");
-      cleanup();
-     } 
-   /* clear working array */
+   {fprintf(stderr,"allocation problems\n");
+   cleanup();
+   } 
+ /* clear working array */
  for (i=0;i<volume;i++)
    work[i]=0;
-   /* allocate a few bytes of extra space at ends of field array */
+ /* allocate a few bytes of extra space at ends of field array */
  for (i=0;i<2;i++)
-  {if (NULL!=field[i])  free((char *) field[i]-BPW);
+   {if (NULL!=field[i])  free((char *) field[i]-BPW);
    if (NULL==( field[i]= (unsigned char *) malloc(volume+2*BPW)))
      {fprintf(stderr,"allocation problems\n");
-      cleanup();
+     cleanup();
      } 
    /* clear initial state */
    for (j=0;j<(volume+2*BPW);j++)
      field[i][j]=0;
    field[i]+=BPW;
-  } 
-  /* set blockshift to log_2 of block */
+   } 
+ /* set blockshift to log_2 of block */
  blockshift=0;
  i=block;
  while(i>>=1)
-  blockshift++; 
-  /* draw everything */
+   blockshift++; 
+ /* draw everything */
  repaint();
  return;
 }
 
-Window makebutton(xoffset,yoffset,xsize,ysize)
-int xoffset,yoffset,xsize,ysize;
+Window makebutton(int xoffset, int yoffset, int xsize, int ysize)
 /* Puts button of specified dimensions on window.  Nothing is drawn. */
 {Window buttonwindow;
  long event_mask;
@@ -947,13 +944,11 @@ int xoffset,yoffset,xsize,ysize;
  return buttonwindow;
 }
 
-void drawbutton(buttonwindow,xoffset,yoffset,xsize,ysize,text,state)
-Window buttonwindow;
-int xoffset,yoffset,xsize,ysize,state;
-char * text;
+void drawbutton(Window buttonwindow, int xoffset, int yoffset,
+     int xsize, int ysize, char *text, int state)
 /* Draw a button in buttonwindow of specified dimensions with text
    centered.  Color of button determined by sign of "state." 
-   size of border determined by magnitude. */
+   size of border determined by magnitude of state. */
 {int textlength,i,j;
  int cdark,clight,cup,cdown;
  int cleft,cright,cbutton,ctext;
@@ -1095,9 +1090,9 @@ int xsize,ysize;          /* output picture dimensions    */
  return;
 }
 
-void savepic(data,xsize,ysize)  /* save the field as a GIF image */
-unsigned char *data;       /* where the input data starts   */
-int xsize,ysize;           /* picture dimensions            */
+void savepic(unsigned char *data, int xsize, int ysize)  
+  /* save the field as a GIF image */
+  /* data points to where the input data starts   */
 {int i;
  int colorbits=5,codesize=5; /* assume a 32 color image */
  FILE *outfile;
@@ -1224,7 +1219,7 @@ int size;         /* how much data                */
     /* look for an unstored string */
    length=1;
    /* the LZW stuff */
-   LZW (nextcode>
+   while (nextcode>
             (currentcode=findcode(length,(char *)(ptr+currentplace))))
     {prefixcode=currentcode;
      length++;
